#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <int.h>
#include "cornet.h"

/*---- variable -----------------------------------------------*/

net_t net;        // ࠬ 

/*---- code ---------------------------------------------------*/


/**********************/
/* 뢮  slave */
/**********************/
void net_adr (uchar adr)
{
   sio_putcw (net.port, NET_DLE);
   sio_putcw (net.port, adr);
   net.obyte += 2;
   sio_oflush (net.port);
}


/*****************************/
/* 뢮    */
/*****************************/
void net_wr (uchar byte)
{
   sio_putcw (net.port, byte);
   if (byte == NET_DLE) { sio_putcw (net.port, NET_DLE); }
   net.obyte++;
}


/*****************************************/
/* 뢮      CRC-16 */
/*****************************************/
void net_blkwr (char *bf, uint sz)
{
   uint i;

   crc_init();

   net.o = 0;
   for (i=0; i<sz; i++) {
      net_wr (bf[i]);
      crc_add (bf[i]);
      net.o++;
      }

   sio_oflush (net.port);
   net_wr (Hi(crc));
   net_wr (Lo(crc));
   sio_oflush (net.port);
}


/******************************/
/* ⥭    */
/******************************/
int net_rd (ulong tout)
// tout - ᨬ ⠩, uS
{
   int rc;

   rc = sio_tgetc (net.port, tout);
   if (rc < 0) goto ex;

   if ((uchar)rc == NET_DLE) {
      rc = sio_tgetc (net.port, tout);
      if (rc < 0) goto ex;
      }
   net.ibyte++;
ex:
   return (rc);      // rc < 0: timeout;
}


/******************************************/
/* ⥭      CRC-16 */
/******************************************/
int net_blkrd (char *bf, uint sz, ulong tout)
// tout - ᨬ ⠩, uS
{
   uint i;
   int rc;

   crc_init();
   net.i = 0;

   sio_ipurge (net.port);     // ⨬ 室  ( 뢮  )

   for (i=0; i<sz; i++) {
      rc = net_rd(tout);
      if (rc < 0) { rc = 2; goto ex; }
      bf[i] = (uchar)rc;
      crc_add ((uchar)rc);
      net.i++;
      }

   rc = net_rd(tout);
   if (rc < 0) { rc = 2; goto ex; }
   net.i++;
   i = ((uint)rc << 8);
   rc = net_rd(tout);
   if (rc < 0) { rc = 2; goto ex; }
   net.i++;
   i |= (uchar)((uint)rc);
   rc = (crc != i);

ex:
   return (rc);      // 0 - crc ok; 1 - crc error; 2 - timeout
}


/********************************************/
/* ନ஢ ॠ  砫 ᨨ */
/********************************************/
void net_preamble (ulong tx)
{
   uchar far tf;

   tout_init(tx, &tf);

   net_tx_en();                        // set rs485 driver to transmit
   do {
      net_adr (NET_ADR_MASTER);
      } while (tf==0);
   net_tx_dis();                       // set rs485 driver to receive

   tout_cancel();
}


/*******************************************/
/* ନ஢  뢮   */
/*******************************************/
void net_cmd (uchar adr, uchar cmd, uint param)
{
   uchar bf[4];

   bf[0] = cmd;
   bf[1] = Hi(param);
   bf[2] = Lo(param);
   bf[3] = adr;

   delay (mS(3));                      // 쭠 㧠  
   net_tx_en();                        // set rs485 driver to transmit
   net_adr (adr);                      //  slave
   net_blkwr (bf, 4);                  // 4    2  CRC-16
   net_tx_dis();                       // set rs485 driver to receive
}


/***************************************/
/*    slave   */
/***************************************/
void net_write (uchar adr, char *bf, uint sz)
{
   uchar cbf[4];

   cbf[0] = NET_CMD_WR;
   cbf[1] = Hi(sz);
   cbf[2] = Lo(sz);
   cbf[3] = adr;

   delay (mS(3));                      // 쭠 㧠  
   net_tx_en();
   net_adr (adr);                      //  slave
   net_blkwr (cbf, 4);                 // 4    2  CRC-16
   delay (mS(1));
   net_blkwr (bf, sz);
   net_tx_dis();
   net.oblk++;
}


/****************************************/
/*  ⥭  slave   */
/****************************************/
int net_read (uchar adr, char *bf, uint sz)
{
   int rc, i;

   for (i=0; i<4; i++) {               //  訡   4 ࠧ
      net_cmd (adr, NET_CMD_RD, sz);
      rc = net_blkrd (bf, sz, (NET_TOUT_RCV/4));
      if (rc == 0) break;
      }

   switch (rc) {
      case 0 : net.iblk++; break;
      case 1 : net.crc++;  break;
      case 2 : net.tout++; break;
      }

   return (rc);   // 0 - crc ok; 1 - crc error; 2 - timeout
}


/********************************/
/*  ⥭  slave */
/********************************/
int net_st (uchar adr, ulong tout, uint rpt)
// tout - ᨬ ⠩  ਥ, uS
// rpt  - ᫮ ஢  訡
{
   int rc, i;
   char bf[2+2];

   for (i=0; i<rpt; i++) {       //  訡  rpt ࠧ
      net_cmd (adr, NET_CMD_STAT, 0);
      rc = net_blkrd (bf, 2, tout);

      if (rc == 0) {
         net.stio = bf[0];
         net.stcmd = bf[1];
         break;
         }
      }

   return (rc);      // 0 - crc ok; 1 - crc error; 2 - timeout
}


/**************************************************/
/*  襭 믮   slave */
/**************************************************/
int net_wait_rdy (uchar adr)
{
   int i, rc;

   #define  RDY_NMAX  500

   for (i=0; i<RDY_NMAX; i++) {
      rc = net_st (adr, mS(100), 20);
      if (rc) break;
      if ((net.stcmd & NET_ST_BSY) == 0) break;
      }
   if (i==RDY_NMAX) rc = 2;

   #undef   RDY_NMAX

   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
}


/*********************************************/
/* 믮      */
/*********************************************/
int net_cmd_get (uchar adr, uchar cmd, uint param, char *bf, uint sz)
{
   int rc;

   net_cmd (adr, cmd, param);
   rc = net_wait_rdy (adr);      //  襭 믮 
   if (rc == 0) {
      if (net.stcmd & NET_ST_EXEERR) {
         rc = 3;
         }
      else {
         rc = net_read (adr, bf, sz);
         }
      }

   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
                     // 3 - 訡 믮 
}


/*********************************************/
/*     믮   */
/*********************************************/
int net_cmd_put (uchar adr, uchar cmd, uint param, char *bf, uint sz)
{
   int rc;

   net_write (adr, bf, sz);
   net_cmd (adr, cmd, param);
   rc = net_wait_rdy (adr);      //  襭 믮 

   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
}



/*****************************************/
/*  ᨬ    slave */
/*****************************************/
int net_putkey (uchar adr, uint c)
{
   int rc, i;

   net_cmd (adr, NET_CMD_KEYB, c);
   rc = net_wait_rdy (adr);
   if (rc == 0) {
      if (net.stio || net.stcmd) {
         rc = 1;
         }
      }
   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
}


/******************************************************/
/*   slave ࠧ ࠬ 䨣樨 */
/******************************************************/
int net_getcfg (uchar adr, uchar subcmd, char *bf, uchar m)
// subcmd = SL_CFG_*
// m - 0: get, no reverse; 1: get, reverse; 2: reverse bf, no get
{
   int rc;

   static uint s_info[] = {   // ᫮  ࠧ ⮢  sl_info_t
      120,1, 120,1, 1,2, 0
      };

   static uint s_tem[] = {    // ᫮  ࠧ ⮢  sl_temcfg_t
      1,1, 1,2, 1,1, 3,4, 1,1, 4,2, 2,4, 0
      };

   static uint s_press[] = {  // ᫮  ࠧ ⮢  sl_presscfg_t
      2,1, 5,4, 4,2, 2,4,
      8,2,                          // Tt[PR_TSZ_T]
      64,2,                         // Tu[PR_TSZ_T][PR_TSZ_U]
      64,2,                         // Tp[PR_TSZ_T][PR_TSZ_U]

      1,2, 2,1, 1,2, 1,1, 1,2, 1,4, // pFu[PR_TSZ_T]
      1,2, 2,1, 1,2, 1,1, 1,2, 1,4,
      1,2, 2,1, 1,2, 1,1, 1,2, 1,4,
      1,2, 2,1, 1,2, 1,1, 1,2, 1,4,
      1,2, 2,1, 1,2, 1,1, 1,2, 1,4,
      1,2, 2,1, 1,2, 1,1, 1,2, 1,4,
      1,2, 2,1, 1,2, 1,1, 1,2, 1,4,
      1,2, 2,1, 1,2, 1,1, 1,2, 1,4,

      1,2, 2,1, 1,2, 1,1, 1,2, 1,4, // F
      0
      };

   static uint s_th[] = {     // ᫮  ࠧ ⮢  sl_thcfg_t
      1,4,
      32,2,                         // T[VOL_TSZ]
      32,2,                         // N[VOL_TSZ]
      1,2, 2,1, 1,2, 1,1, 1,2, 1,4, // Ftbl
      0
      };

   static uint s_pause[] = {  // ᫮  ࠧ ⮢  sl_pausecfg_t
      1,4, 1,1, 0
      };

   static uint s_sys[] = {    // ᫮  ࠧ ⮢  sl_syscfg_t
      1,1, 1,4, 3,1, 2,2, 2,1, 0
      };

   static uint s_sio[] = { 0 };    //  sl_siocfg_t  ࠬ஢  樨

   static uint s_out[] = {    // ᫮  ࠧ ⮢  sl_outcfg_t
      1,1, 1,2, 2,1, 2,4, 0
      };

   static uint s_cst[] = {    // ᫮  ࠧ ⮢  sl_cst_t
      4,4, 0
      };

   static uint s_dtj[] = { 0 };    //  sl_dtj_t  ࠬ஢  樨

   static uint sz[] = {       // ࠧ     subcmd
      sizeof(sl_info_t), sizeof(sl_temcfg_t), sizeof(sl_presscfg_t),
      sizeof(sl_thcfg_t), sizeof(sl_pausecfg_t), sizeof(sl_syscfg_t),
      sizeof(sl_siocfg_t), sizeof(sl_outcfg_t), sizeof(sl_cst_t),
      sizeof(sl_dtj_t)
      };

   static uint *x[] = {
      s_info, s_tem, s_press, s_th,  s_pause,
      s_sys,  s_sio, s_out,   s_cst, s_dtj
      };



   if (subcmd >= Size(sz, uint)) {
      rc = 3;
      goto ex;
      }

   if (m == 2) { rc = 0; }
   else {   // m == 0,1
      rc = net_cmd_get (adr, NET_CMD_GETCFG, subcmd, bf, sz[subcmd]);
      }

   if ((rc == 0) && m) {
      reverse_struct (bf, x[subcmd]);  // convert to low endian
      }
ex:
   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
                     // 3 - 訡 믮 
}


/***************************************/
/*   slave ⥪騥 ࠬ */
/***************************************/
int net_current (uchar adr, char *bf)
{
   int rc;

   static uint s_curr[] = {   // ᫮  ࠧ ⮢  sl_curr_t
      6,1, 1,2, 1,1,
      3,4, 1,2,
      1,4, 1,2, 1,4,
      1,1, 1,4, 6,1, 1,2, 1,1, 6,1, 1,2, 1,1, 2,4,
      4,4,
      6,1, 1,2, 1,1, 4,4, 2,4, 2,4,
      6,1, 1,2, 1,1, 6,1, 1,2, 1,1, 5,4, 2,4, 2,4,
      6,4,
      1,2, 1,1,
      2,1, 6,1, 1,2, 1,1,
      2,1, 6,1, 1,2, 1,1,
      2,1, 6,1, 1,2, 1,1,
      2,1, 6,1, 1,2, 1,1,
      0
      };

   rc = net_cmd_get (adr, NET_CMD_CURRENT, 0, bf, sizeof(sl_curr_t));
   if (rc == 0) {
      reverse_struct (bf, s_curr);     // convert to low endian
      }
ex:
   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout

}


/************************************************/
/*   slave ⥪饥 ﭨ । */
/************************************************/
int net_queue (uchar adr, char *bf)
{
   int rc;
   static uint s_que[] = {   // ᫮  ࠧ ⮢  sl_queue_t
      4,2, 1,1, 1,2,
      4,2, 1,1, 1,2,
      4,2, 1,1, 1,2,
      4,2, 1,1, 1,2,
      4,2, 1,1, 1,2,
      4,2, 1,1, 1,2,
      0
      };

   rc = net_cmd_get (adr, NET_CMD_QUEUE, 0, bf, sizeof(sl_queue_t));
   if (rc == 0) {
      reverse_struct (bf, s_que);     // convert to low endian
      }

   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
                     // 3 - 訡 믮 
}


/********************************************/
/*   slave    ਨ */
/********************************************/
int net_hist (uchar adr, uchar hcmd, int r, char *bf)
// hcmd -   ਨ:
//    NET_CMD_HISTM, NET_CMD_HISTH, NET_CMD_HISTD, NET_CMD_HISTMN
// r -  : 0 - ᫥, 1 - ।᫥, ...
{
   int rc;
   static uint s_gaz[] = {  // ᫮  ࠧ ⮢  gaz_t
      6,1, 1,2, 1,1,
      4,4, 2,4, 2,4,
      0
      };

   rc = net_cmd_get (adr, hcmd, r, bf, sizeof(gaz_t));
   if (rc == 0) {
      reverse_struct (bf, s_gaz);     // convert to low endian
      }

   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
                     // 3 - 訡 믮 
}


/*****************************************/
int net_hist_m (uchar adr, int r, char *bf) { return (net_hist(adr, NET_CMD_HISTM, r, bf)); }
int net_hist_h (uchar adr, int r, char *bf) { return (net_hist(adr, NET_CMD_HISTH, r, bf)); }
int net_hist_d (uchar adr, int r, char *bf) { return (net_hist(adr, NET_CMD_HISTD, r, bf)); }
int net_hist_mn (uchar adr, int r, char *bf) { return (net_hist(adr, NET_CMD_HISTMN, r, bf)); }


/****************************************/
/*      ᮡ⨩ */
/****************************************/
int net_elog (uchar adr, int r, char *bf)
// r -  : 0 - ᫥, 1 - ।᫥, ...
{
   int rc;
   static uint s_evt[] = {  // ᫮  ࠧ ⮢  evtlog_t
      6,1, 1,2, 1,1,
      1,1, 1,4,
      0
      };

   rc = net_cmd_get (adr, NET_CMD_LOGEV, r, bf, sizeof(evtlog_t));
   if (rc == 0) {
      reverse_struct (bf, s_evt);     // convert to low endian
      }

   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
                     // 3 - 訡 믮 
}


/******************************************/
/*       */
/******************************************/
int net_clog (uchar adr, int r, char *bf)
// r -  : 0 - ᫥, 1 - ।᫥, ...
{
   int rc;
   static uint s_chg[] = {  // ᫮  ࠧ ⮢  chglog_t
      6,1, 1,2, 1,1,  1,1,  0
      };

      // ᫮  ࠧ ⮢   ࠬ஢ chglog_t (  id)
   static uint s_00[] = { 0 };
   static uint s_01[] = { 0 };
   static uint s_02[] = { 0 };
   static uint s_03[] = { 4,2,0 };
   static uint s_04[] = { 0 };
   static uint s_05[] = { 2,4,0 };
   static uint s_06[] = { 2,4,0 };
   static uint s_07[] = { 2,4,0 };
   static uint s_08[] = { 0 };
   static uint s_09[] = { 0 };
   static uint s_10[] = { 2,4,0 };
   static uint s_11[] = { 2,4,0 };
   static uint s_12[] = { 0 };
   static uint s_13[] = { 2,4,0 };
   static uint s_14[] = { 2,4,0 };
   static uint s_15[] = { 2,4,0 };
   static uint s_16[] = { 0 };
   static uint s_17[] = { 0 };
   static uint s_18[] = { 0 };
   static uint s_19[] = { 4,2,0 };
   static uint s_20[] = { 4,2,0 };
   static uint s_21[] = { 2,4,0 };
   static uint s_22[] = { 0 };
   static uint s_23[] = { 0 };
   static uint s_24[] = { 0 };
   static uint s_25[] = { 0 };
   static uint s_26[] = { 0 };
   static uint s_27[] = { 0 };
   static uint s_28[] = { 0 };
   static uint s_29[] = { 2,4,0 };
   static uint s_30[] = { 2,4,0 };
   static uint s_31[] = { 2,4,0 };
   static uint s_32[] = { 2,4,0 };
   static uint s_33[] = { 0 };
   static uint s_34[] = { 0 };
   static uint s_35[] = { 0 };
   static uint s_36[] = { 2,4,0 };
   static uint s_37[] = { 2,4,0 };
   static uint s_38[] = { 4,2,0 };
   static uint s_39[] = { 4,2,0 };
   static uint s_40[] = { 4,2,0 };
   static uint s_41[] = { 4,2,0 };
   static uint s_42[] = { 4,2,0 };
   static uint s_43[] = { 4,2,0 };
   static uint s_44[] = { 2,4,0 };
   static uint s_45[] = { 0 };
   static uint s_46[] = { 0 };
   static uint s_47[] = { 0 };
   static uint s_48[] = { 0 };
   static uint s_49[] = { 0 };
   static uint s_50[] = { 0 };
   static uint s_51[] = { 0 };
   static uint s_52[] = { 2,4,0 };
   static uint s_53[] = { 2,4,0 };

   static uint *s_prm[] = {
      s_00, s_01, s_02, s_03, s_04, s_05, s_06, s_07, s_08, s_09,
      s_10, s_11, s_12, s_13, s_14, s_15, s_16, s_17, s_18, s_19,
      s_20, s_21, s_22, s_23, s_24, s_25, s_26, s_27, s_28, s_29,
      s_30, s_31, s_32, s_33, s_34, s_35, s_36, s_37, s_38, s_39,
      s_40, s_41, s_42, s_43, s_44, s_45, s_46, s_47, s_48, s_49,
      s_50, s_51, s_52, s_53,
      NULL
      };

   chglog_t *p;

   rc = net_cmd_get (adr, NET_CMD_LOGCH, r, bf, sizeof(chglog_t));
   if (rc == 0) {
      reverse_struct (bf, s_chg);     // convert to low endian
      p = (chglog_t *)bf;
      reverse_struct (p->bf, s_prm[p->id]);
      }

   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
                     // 3 - 訡 믮 
}



/**************************************/
/*  n ᥩ    */
/**************************************/
int net_getlog (uchar adr, uchar log, int r0, int n, char *bf)
// log - ⨯  (SL_LOG_*)
// r0  - 砫쭠 : 0 - ᫥, 1 - ।᫥, ...
// n   - ⢮ 뢠 ᥩ
{
   int i, k, rc, sz;
   char *p;
   xque_t *pq;
   int (*pF)(uchar, int, char *);
   static sl_queue_t slq;
   static int recsz[] = {
      sizeof(gaz_t), sizeof(gaz_t), sizeof(gaz_t), sizeof(gaz_t),
      sizeof(evtlog_t), sizeof(chglog_t)
      };
   static xque_t *qa[] = {
      &slq.hm, &slq.hh, &slq.hd, &slq.hmn, &slq.le, &slq.lc
      };
   static int (*F[])(uchar, int, char *) = {
      net_hist_m, net_hist_h, net_hist_d, net_hist_mn, net_elog, net_clog
      };

      // ⥪饥 ﭨ ।
   rc = net_queue (adr, (char *)(&slq));
   if (rc) goto ex;

   sz = recsz[log];
   pq = qa[log];
   pF = F[log];

      // ࠭稢 r0, ᫨  ।  ᥩ
   if (r0 >= pq->q) { r0 = pq->q - 1; }

   p = bf;
   for (i=0; (i<n) && (i<=r0); i++) {
      rc = (*pF)(adr, r0-i, p);
      if (rc) goto ex;
      p += sz;
      }

   if (i != 0) {
         // ஢塞    ﭨ ।
      k = pq->i;
      rc = net_queue (adr, (char *)(&slq));
      if (rc) goto ex;
         // ᫨ , ⮣ 뢠  ࠧ  ᫥ 
      if (pq->i != k) {
         if (i > 1) {
               // 㤠塞 ᠬ  
            memmove (bf, bf+sz, i*sz);
               // 塞 ।᫥ 
            rc = (*pF)(adr, 1, bf+((i-2)*sz));
            if (rc) goto ex;
            }
            // 塞 ᫥ 
         rc = (*pF)(adr, 0, bf+((i-1)*sz));
         if (rc) goto ex;
         }
      }

ex:
   net.rec = ((rc)? 0 : i);   // ᫮ ⠭ ᥩ: 0...n
   net.r0  = r0;              //  砫쭮 ⠭  (r0  ࠭稢)

   return (rc);      // 0 - ok; 1 - crc error; 2 - timeout
                     // 3 - 訡 믮 
}


